#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
filter_illumina_index Unit Tests
========================

Performs the following tests:
    - Loading the script with no options
    - Loading the script with --version
    - Running the script with various options using a test input file. The
      output of the script is compared line-by-line to reference output files.

To run tests, use one of these commands:
|   python -m unittest discover
|   python -m filter_illumina_index.tests.test_filter_illumina_index

The reference output files can be (re)generated with the following command.
Suggest manually checking these output files (using git to easily see changes)
prior to packaging these in a new version:
|   python -m filter_illumina_index.tests.test_filter_illumina_index --generate

"""

# Currently does the following tests:
#   Script loading and argparse exitcodes
#   Reading fastq and fastq.gz
#   Mismatch counting by comparing summaries (compared to expected)
#     With different index
#   Writing fastq and fastq.gz to filtered/unfiltered (compared to expected)
#     With different index and diff # tolerated mismatches

import unittest
import sys
import functools
import json
import itertools
import gzip

from filter_illumina_index.filter_illumina_index import main as filter_illumina_index_main2
filter_illumina_index_main = functools.partial(filter_illumina_index_main2, return_result = True)


tests_root = 'filter_illumina_index/tests/data/'
tests_results_root = 'filter_illumina_index/tests/data/results/'
tests_output_root = 'var/'
input_test_file_fastq = tests_root + 'test_reads_GATCGTGT.fastq'
input_test_file_fastq_gz = tests_root + 'test_reads_GATCGTGT.fastq.gz'
input_test_file_diffbarcodes = tests_root + 'test_reads_diffbarcodes.fastq'


test_set_exitcodes = [
    # tuples of ([options], expected return code)
    # only for sys.exit by argparse
    ([], 2),
    (['--version'], 0),
    ([input_test_file_fastq], 2),
    (['--index','GATCGTGT'], 2),
]

test_sets_vs_summary = [
    # tuples of ([options], expected output file,  [generate])
    # generate is optional, if False, then the results will not be generated
    # for this test set

    # READ TESTS
    # test read fastq to expected summary, with dnaio -t 1 (default) or -t 0
    ([input_test_file_fastq, '--index','GATCGTGT'],'test_reads_GATCGTGT_results.json'),
    ([input_test_file_fastq, '--index','GATCGTGT','-t','0'],'test_reads_GATCGTGT_results.json',False),
    # test read fastq.gz to expected summary, with dnaio -t 1 (default) or -t 0
    ([input_test_file_fastq_gz, '--index','GATCGTGT'],'test_reads_GATCGTGT_results.json',False),
        # this gives a ResourceWarning for unclosed files in xopen v0.8.4, fixed in v0.9.0
    ([input_test_file_fastq_gz, '--index','GATCGTGT','-t','0'],'test_reads_GATCGTGT_results.json',False),

    # MISMATCH NUMBER TESTS
    # test of mismatch calculation, see examples in README
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-vv'],'test_reads_diffbarcodes_results_TGACCAAT.json'),
    ([input_test_file_diffbarcodes, '--index','NNNCCAAT','-vv'],'test_reads_diffbarcodes_results_NNNCCAAT.json'),

    # MISMATCH FILTER/UNFILTER TESTS
    # test number of filtered/unfiltered with diff mismatch tolerances
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-vv','-m 1'],'test_reads_diffbarcodes_results_TGACCAAT_m1.json'),
    ([input_test_file_diffbarcodes, '--index','NNNCCAAT','-vv','-m 1'],'test_reads_diffbarcodes_results_NNNCCAAT_m1.json'),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-vv','-m 2'],'test_reads_diffbarcodes_results_TGACCAAT_m2.json'),
    ([input_test_file_diffbarcodes, '--index','NNNCCAAT','-vv','-m 2'],'test_reads_diffbarcodes_results_NNNCCAAT_m2.json'),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-vv','-m 3'],'test_reads_diffbarcodes_results_TGACCAAT_m3.json'),
    ([input_test_file_diffbarcodes, '--index','NNNCCAAT','-vv','-m 3'],'test_reads_diffbarcodes_results_NNNCCAAT_m3.json'),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-vv','-m 4'],'test_reads_diffbarcodes_results_TGACCAAT_m4.json'),
    ([input_test_file_diffbarcodes, '--index','NNNCCAAT','-vv','-m 4'],'test_reads_diffbarcodes_results_NNNCCAAT_m4.json'),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-vv','-m 5'],'test_reads_diffbarcodes_results_TGACCAAT_m5.json'),
    ([input_test_file_diffbarcodes, '--index','NNNCCAAT','-vv','-m 5'],'test_reads_diffbarcodes_results_NNNCCAAT_m5.json'),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-vv','-m 6'],'test_reads_diffbarcodes_results_TGACCAAT_m6.json'),
    ([input_test_file_diffbarcodes, '--index','NNNCCAAT','-vv','-m 6'],'test_reads_diffbarcodes_results_NNNCCAAT_m6.json'),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-vv','-m 7'],'test_reads_diffbarcodes_results_TGACCAAT_m7.json'),
    ([input_test_file_diffbarcodes, '--index','NNNCCAAT','-vv','-m 7'],'test_reads_diffbarcodes_results_NNNCCAAT_m7.json'),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-vv','-m 8'],'test_reads_diffbarcodes_results_TGACCAAT_m8.json'),
    ([input_test_file_diffbarcodes, '--index','NNNCCAAT','-vv','-m 8'],'test_reads_diffbarcodes_results_NNNCCAAT_m8.json'),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-vv','-m 10'],'test_reads_diffbarcodes_results_TGACCAAT_m10.json'),
    ([input_test_file_diffbarcodes, '--index','NNNCCAAT','-vv','-m 10'],'test_reads_diffbarcodes_results_NNNCCAAT_m10.json'),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-vv','-m 20'],'test_reads_diffbarcodes_results_TGACCAAT_m20.json'),
    ([input_test_file_diffbarcodes, '--index','NNNCCAAT','-vv','-m 20'],'test_reads_diffbarcodes_results_NNNCCAAT_m20.json'),
]

test_sets_vs_output = [
    # tuples of ([options], (expected filtered file, expected unfiltered file), [generate])
    # either filtered/unfiltered can be None in which not checked against
    # end with .gz to automatically test compressed outputs 
    # generate is optional, if False, then the results will not be generated
    # for this test set
    ([input_test_file_fastq, '--index','GATCGTGT',],
     ('test_reads_GATCGTGT_filtered.fastq','test_reads_GATCGTGT_unfiltered.fastq')
    ),
    ([input_test_file_fastq, '--index','GATCGTGT',],
     (None,'test_reads_GATCGTGT_unfiltered.fastq'), False
    ),
    ([input_test_file_fastq, '--index','GATCGTGT',],
     ('test_reads_GATCGTGT_filtered.fastq', None), False
    ),
    ([input_test_file_fastq, '--index','GATCGTGT',],
     ('test_reads_GATCGTGT_filtered.fastq.gz','test_reads_GATCGTGT_unfiltered.fastq.gz')
    ),
    ([input_test_file_fastq, '--index','GATCGTCT',], # inverted, GATCGTCT is mismatch
     ('test_reads_GATCGTGT_unfiltered.fastq.gz','test_reads_GATCGTGT_filtered.fastq.gz'), False
    ),

    # tests with mismatches allowed
    ([input_test_file_fastq, '--index','GATCGTGT','-m','1'], # allow 1 mismatch
     ('test_reads_GATCGTGT_filtered_m1.fastq.gz','test_reads_GATCGTGT_unfiltered_m1.fastq')
    ),
    ([input_test_file_fastq, '--index','GATCGTCT','-m','1'], # inverted, GATCGTCT is mismatch, but allow 1 mismatch
     ('test_reads_GATCGTGT_filtered_m1.fastq.gz','test_reads_GATCGTGT_unfiltered_m1.fastq'), False
    ),

    # tests with varyinf # mismatches allowed
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-m','0','-vv'],
     ('test_reads_diffbarcodes_filtered_TGACCAAT_m0.fastq.gz',
      'test_reads_diffbarcodes_unfiltered_TGACCAAT_m0.fastq.gz'), 
    ),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-m','1','-vv'],
     ('test_reads_diffbarcodes_filtered_TGACCAAT_m1.fastq.gz',
      'test_reads_diffbarcodes_unfiltered_TGACCAAT_m1.fastq.gz'), 
    ),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-m','2','-vv'],
     ('test_reads_diffbarcodes_filtered_TGACCAAT_m2.fastq.gz',
      'test_reads_diffbarcodes_unfiltered_TGACCAAT_m2.fastq.gz'), 
    ),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-m','3','-vv'],
     ('test_reads_diffbarcodes_filtered_TGACCAAT_m3.fastq.gz',
      'test_reads_diffbarcodes_unfiltered_TGACCAAT_m3.fastq.gz'), 
    ),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-m','5','-vv'],
     ('test_reads_diffbarcodes_filtered_TGACCAAT_m5.fastq.gz',
      'test_reads_diffbarcodes_unfiltered_TGACCAAT_m5.fastq.gz'), 
    ),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-m','8','-vv'],
     ('test_reads_diffbarcodes_filtered_TGACCAAT_m8.fastq.gz',
      'test_reads_diffbarcodes_unfiltered_TGACCAAT_m8.fastq.gz'), 
    ),
    ([input_test_file_diffbarcodes, '--index','TGACCAAT','-m','10','-vv'],
     ('test_reads_diffbarcodes_filtered_TGACCAAT_m10.fastq.gz',
      'test_reads_diffbarcodes_unfiltered_TGACCAAT_m10.fastq.gz'), 
    )

]

class TestFilerIlluminaIndex(unittest.TestCase):
    def test_exitcodes(self):
        for test_set in test_set_exitcodes:
            test_options, test_expected_exitcode = test_set
            flat_test_options = " ".join(test_options)
            with self.subTest(options = flat_test_options,
                              exitcode = test_expected_exitcode):
                print("Testing options {}, expecting exitcode {}:".format(
                    flat_test_options,
                    test_expected_exitcode))
                with self.assertRaises(SystemExit) as cm:
                    filter_illumina_index_main(test_options)
                exitcode = cm.exception.code
                self.assertEqual(exitcode, test_expected_exitcode, 
                    'Unexpected return code: got {}, expected {}'.format(exitcode, test_expected_exitcode))

    def test_results_summary(self):
        # generic tests compared to saved summary
        for test_set in test_sets_vs_summary:
            test_options, test_expected_output_file = test_set[:2]
            flat_test_options = " ".join(test_options)
            test_expected_output_path = tests_results_root + test_expected_output_file
            with self.subTest(options = flat_test_options,
                              reference_file = test_expected_output_file):
                print("Testing options {}, comparing summary to {}:".format(
                    flat_test_options,
                    test_expected_output_file))
                test_result = filter_illumina_index_main(test_options)
                test_json_safe = json.loads(json.dumps(test_result))
                    # convert to json and back to ensure it is comparable to jsonified expected results
                with open(test_expected_output_path,'r') as expected_obj:
                    expected_result = json.load(expected_obj)
                self.assertEqual(test_json_safe, expected_result,'Mismatch results {} vs {}'.format(
                    test_json_safe, expected_result))

    def helper_compare_files(self, path1, path2):
        opener1 = functools.partial(gzip.open, mode='rt') if path1.endswith('gz') else functools.partial(open, mode='r')
        opener2 = functools.partial(gzip.open, mode='rt') if path2.endswith('gz') else functools.partial(open, mode='r')
        with opener1(path1) as handle1:
            with opener2(path2) as handle2:
                for line_index,(line1,line2) in enumerate(
                    itertools.zip_longest(handle1, handle2),1):
                    self.assertEqual(line1.strip('\r\n'),
                        line2.strip('\r\n'),
                        '{} vs {}, line {} mismatch'.format(path1,
                            path2, line_index))

    def test_results_output(self):
        for test_set in test_sets_vs_output:
            test_options, test_expected_output_pair = test_set[:2]
            flat_test_options = " ".join(test_options)
            test_expected_filtered_file, test_expected_unfiltered_file = test_expected_output_pair
            if test_expected_filtered_file is not None:
                test_expected_filtered_path = tests_results_root + test_expected_filtered_file
                test_output_filtered_path = tests_output_root + test_expected_filtered_file
                test_options.extend(['-f',test_output_filtered_path])
            if test_expected_unfiltered_file is not None:
                test_expected_unfiltered_path = tests_results_root + test_expected_unfiltered_file
                test_output_unfiltered_path = tests_output_root + test_expected_unfiltered_file
                test_options.extend(['-u',test_output_unfiltered_path])
            with self.subTest(options = flat_test_options,
                              filtered_file = test_expected_filtered_file,
                              unfiltered_file = test_expected_unfiltered_file,
                              ):
                print("Testing options {}, comparing output to filtered {} / unfiltered{}:".format(
                    flat_test_options,
                    test_expected_filtered_file,
                    test_expected_unfiltered_file))
                filter_illumina_index_main(test_options)
                if test_expected_filtered_file:
                    self.helper_compare_files(test_expected_filtered_path, test_output_filtered_path)
                if test_expected_unfiltered_file:
                    self.helper_compare_files(test_expected_unfiltered_path, test_output_unfiltered_path)


# this function will generate results files, instead of checking against them
# will be run if called with option --generate
def generate_results():
    for test_set in test_sets_vs_summary:
        if len(test_set)>2: # third parameter == False, don't generate
            if test_set[2]  == False:
                continue
        test_options, test_expected_output_file = test_set[:2]
        test_expected_output_path = tests_results_root + test_expected_output_file
        print("Running with options {}, saving to {}:".format(' '.join(test_options), 
            test_expected_output_path))
        results = filter_illumina_index_main(test_options)
        with  open(test_expected_output_path,'w') as test_expected_output_handle:
            json.dump(results, test_expected_output_handle, indent=1)
            
    for test_set in test_sets_vs_output:
        if len(test_set)>2: # third parameter == False, don't generate
            if test_set[2]  == False:
                continue
        test_options, test_expected_output_pair = test_set[:2]
        test_expected_filtered_file, test_expected_unfiltered_file = test_expected_output_pair
        if test_expected_filtered_file is not None:
            test_expected_filtered_path = tests_results_root + test_expected_filtered_file
            test_options.extend(['-f',test_expected_filtered_path])
        if test_expected_unfiltered_file is not None:
            test_expected_unfiltered_path = tests_results_root + test_expected_unfiltered_file
            test_options.extend(['-u',test_expected_unfiltered_path])
        print("Running with options {}:".format(' '.join(test_options)))
        filter_illumina_index_main(test_options)


if __name__ == '__main__':
    if '--generate' in sys.argv:
        do_generate = True
    else:
        do_generate = False
    if do_generate: generate_results()
    else: unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import functools
import mimetypes
import gzip
import sys
from functools import partial

import dnaio
import xopen

"""
Filter a Illumina FASTQ file based on index sequence.

Reads a Illumina FASTQ file and compares the sequence index in the
`sample number` position of the sequence identifier to a supplied sequence
index. Entries that match the sequence index are filtered into the *filtered
file* (if any) and entries that don't match are filtered into the *unfiltered
file* (if any). Displays the count of total, filtered and unfiltered reads,
as well as the number of mismatches found across all reads. Matching tolerating
a certain number of mismatches (`-m` parameter), and gzip compression for input
(detected on the basis of file extension) and output (specified using `-c`
parameter) are supported.
"""
_PROGRAM_NAME = 'filter_illumina_index'
# -------------------------------------------------------------------------------
# Author:       Tet Woo Lee
#
# Created:      2018-12-13
# Copyright:    © 2018 Tet Woo Lee
# Licence:      GPLv3
#
# Dependencies: dnaio, tested with v0.4.1
#               xopen, tested with v0.9.0
# -------------------------------------------------------------------------------

_PROGRAM_VERSION = '1.0.4.dev2'
# -------------------------------------------------------------------------------
# ### Change log
#
# version 1.0.4 2020-01-04
# Speed up and algorithm changes
#   - Switch to `dnaio` over Biopython to improve speed (>3x faster + multi-
#     threading support for compression)
#   - Change mismatch calculation algorithm, now includes any characters
#     missing in filter-by index or read index
#
# version 1.0.3.post2 2020-01-04
# Improved output
#   - Add version number to output
#   - Show parameters in output
#   - Allow no argument for passthrough mode
#
# version 1.0.3.post1 2020-01-04
# Minor bugfix
#   - Bugfix: Bump version number in script
#
# version 1.0.3 2020-01-04
# Added `passthrough` mode with empty index
#
# version 1.0.2 2018-12-19
# Shows statistics on number of mismatches found
#
# version 1.0.1 2018-12-19
# Speed up number of mismatches calculation
#
# version 1.0 2018-12-14
# Minor updates for PyPi and conda packaging
#
# version 1.0.dev1 2018-12-13
# First working version
#
# -------------------------------------------------------------------------------

# TODO: tests

# INITIALISATION

def main(argv = None, return_result = False):
        # return_result = True will return summary of output to caller (for testing)
    if argv is None: argv = sys.argv[1:] # if parameters not provided, use sys.argv

    _PROGRAM_NAME_VERSION = '{} {}'.format(_PROGRAM_NAME, _PROGRAM_VERSION)
    parser = argparse.ArgumentParser(prog=_PROGRAM_NAME,
                                     description=__doc__,
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_required_named = parser.add_argument_group('required named arguments')
    parser.add_argument('--version', action='version',
                        version=_PROGRAM_NAME_VERSION)
    parser.add_argument('inputfile',
                        help='Input FASTQ file, compression (`.gz`, `.bz2` and '
                        '`.xz`) supported')
    parser.add_argument('-f', '--filtered',
                        help='Output FASTQ file containing filtered (positive) reads; '
                        'compression detected by extension')
    parser.add_argument('-u', '--unfiltered',
                        help='Output FASTQ file containing unfiltered (negative) reads; '
                        'compression detected by extension')
    parser_required_named.add_argument('-i', '--index', required=True, const='', nargs='?',
                                       help='Sequence index to filter for; if empty '
                                       '(i.e. no argument or "") then program will '
                                       'run in "passthrough" mode with all reads '
                                       'directed to filtered file with no processing')
    parser.add_argument('-m', '--mismatches', default=0, type=int,
                        help='Maximum number of mismatches to tolerate')
    parser.add_argument('-t', '--threads', default=1, type=int,
                        help='Number of threads to pass to `xopen` for each '
                        'open file; use 0 to turn off `pigz` use and rely '
                        'on `gzip.open` so no extra threads spawned.')
    parser.add_argument('-l', '--compresslevel', default=6, type=int, choices=range(1,10),
                        help='Compression level for writing gzip files; '
                        'ignored if gzip compression not used')
    parser.add_argument('-v', '--verbose', action='count', default=0,
                        help='Increase logging verbosity, available levels 1 to 2 '
                             'with `-v` to `-vv`')
    args = parser.parse_args(argv)


    input_path = args.inputfile
    out_filtered_path = args.filtered
    out_unfiltered_path = args.unfiltered
    filter_seq_index = args.index
    max_tolerated_mismatches = args.mismatches
    threads = args.threads
    compresslevel = args.compresslevel
    verbose = args.verbose

    if filter_seq_index == '':
        passthrough_mode = True
        max_tolerated_mismatches = float('NaN')
    else:
        passthrough_mode = False

    # HELPER FUNCTIONS
    print(_PROGRAM_NAME_VERSION)
    print("Input file: {}".format(input_path))
    print("Filtering for sequence index: {}{}".format(filter_seq_index,
        "(passthrough mode)" if passthrough_mode else ""))
    print("Max mismatches tolerated: {}".format(max_tolerated_mismatches))
    print("Output filtered file: {}".format(out_filtered_path))
    print("Output unfiltered file: {}".format(out_unfiltered_path))
    if verbose>=1:
        print("Using {} threads per open file".format(threads))
        print("Compression level: {}".format(compresslevel))
        print("Showing verbose level {} logging".format(verbose))


    xopen_xthreads = functools.partial(xopen.xopen, threads=threads,
                                       compresslevel = compresslevel)

    # PROCESSING
    total_reads = 0
    filtered_reads = 0
    unfiltered_reads = 0
    max_tracked_mismatches = len(filter_seq_index)
    cumul_n_mismatches = [0 for i in range(max_tracked_mismatches + 2)]
    # array for tracking number of mismatches (0-indexlen,>indexlen+1)
    # >indexlen+1 is required because we define mismatch to include extra
    # characters from desired index OR read index, which may be longer
    if passthrough_mode: filtered = True
    with dnaio.open(input_path, mode='r', opener=xopen_xthreads) as input_fastq:
        if out_filtered_path:
            filtered_fastq = dnaio.open(out_filtered_path, mode='w', opener=xopen_xthreads)
        else:
            filtered_fastq = None
        if out_unfiltered_path:
            unfiltered_fastq = dnaio.open(out_unfiltered_path, mode='w', opener=xopen_xthreads)
        else:
            unfiltered_fastq = None
        for record in input_fastq:
            total_reads += 1
            if passthrough_mode:
                # always filtered
                if verbose>=2:
                    seqid = record.name
                    print("{} (passthrough-mode) (filtered)".format(seqid))

            else:
                # Illimina sequence identifier in FASTQ files:
                # see http://support.illumina.com/content/dam/illumina-support/help/BaseSpaceHelp_v2/Content/Vault/Informatics/Sequencing_Analysis/BS/swSEQ_mBS_FASTQFiles.htm
                # @<instrument>:<run number>:<flowcell ID>:<lane>:<tile>:<x-pos>:<y-pos> <read>:<is filtered>:<control number>:<sample number>
                # For the Undetermined FASTQ files only, the sequence observed in the index read is written to the FASTQ header in place of the sample number. This information can be useful for troubleshooting demultiplexing.
                # grab the sequence index, use simplistic method of value after last : for efficiency
                seqid = record.name
                entry_seq_index = seqid[seqid.rfind(':')+1:]
                n_mismatches = abs(len(entry_seq_index)-len(filter_seq_index))
                if entry_seq_index!=filter_seq_index:
                    for c1,c2 in zip(entry_seq_index,filter_seq_index):
                        if c1!=c2: n_mismatches+=1
                filtered = (n_mismatches <= max_tolerated_mismatches)
                if verbose>=2:
                    print("{} -> index {} -> {} mismatches ({})".format(seqid,
                        entry_seq_index, n_mismatches,
                        'filtered' if filtered else 'unfiltered'))
                if n_mismatches>=max_tracked_mismatches:
                    n_mismatches = max_tracked_mismatches+1
                cumul_n_mismatches[n_mismatches] += 1
            if filtered:
                filtered_reads += 1
                if filtered_fastq: filtered_fastq.write(record)
            else:
                unfiltered_reads += 1
                if unfiltered_fastq: unfiltered_fastq.write(record)
        if filtered_fastq: filtered_fastq.close()
        if unfiltered_fastq: unfiltered_fastq.close()


    # OUTPUT
    print("Total reads: {}".format(total_reads))
    if passthrough_mode:
        print("Filtered reads: {} (passthrough-mode)".format(filtered_reads))
    else:
        print("Filtered reads: {}".format(filtered_reads))
        print("Unfiltered reads: {}".format(unfiltered_reads))
        for n_mismatches, cumul_mismatches in enumerate(cumul_n_mismatches):
            print(" Reads with {}{} mismatches: {}".format(
                '' if n_mismatches<= max_tracked_mismatches else '>=',
                n_mismatches, cumul_mismatches))

    if return_result: 
        results = {
            "total" : total_reads,
            "filtered": filtered_reads,
            "unfiltered": unfiltered_reads,
            "mismatches": {n_mismatches : cumul_mismatches 
                          for n_mismatches,cumul_mismatches in enumerate(cumul_n_mismatches)}
        }
        return(results)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="filter_illumina_index",
    version="1.0.4.dev2",
    author="Tet Woo Lee",
    author_email="developer@twlee.nz",
    description="Filter a Illumina FASTQ file based on index sequence",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/twlee79/filter_illumina_index",
    packages=setuptools.find_packages(),
    package_data={'filter_illumina_index.tests': ['data/*.fastq',
                                                  'data/*.fastq.gz',
                                                  'data/results/*.fastq',
                                                  'data/results/*.fastq.gz',
                                                  'data/results/*.json']
                                                  },
    install_requires=[
        'xopen >=v0.9.0',
        'dnaio >=v0.4.1'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': [
            'filter_illumina_index = filter_illumina_index.filter_illumina_index:main',
        ],
    },
    test_suite="filter_illumina_index.tests",
    data_files=[("license", ["LICENSE"]),
                ("testdir_placeholder", ["var/.gitkeep"])
                ],
)
